@@ -5,8 +5,8 @@ import argparse
 import logging
 from pathlib import Path
 from pprint import pprint
+import webbrowser
 
-import toml
 import ripio
 
 
@@ -20,6 +20,7 @@ def set_verbosity(args):
     level = logging.ERROR - 10 * min(args.verbosity, 3)
     if level != logging.ERROR:
         print("Verbosity set to {}".format(logging.getLevelName(level)))
+        print("Try 'ripio -vvv' for even more detail")
     logging.getLogger().setLevel(level)
 
 
@@ -32,14 +33,18 @@ def load_config(args):
     elif home_config.exists():
         config = ripio.Config(home_config)
 
+    logging.debug("Loading config '{}'".format(config.fname))
+
     try:
-        for key in ['credentials', 'destdir']:
+        for key in ['destdir', 'bitbucket', 'github']:
             try:
-                setattr(args, key, getattr(config, key)())
+                setattr(args, key, getattr(config, key))
             except ripio.MissingConfig:
                 pass
 
-        logging.debug("Config '{}' loaded".format(args.config))
+        args.credentials = {
+            'bitbucket': config.get_credentials('bitbucket'),
+            'github':    config.get_credentials('github')}
 
     except FileNotFoundError as e:
         logging.error(e)
@@ -50,10 +55,6 @@ def canceled():
     sys.exit(1)
 
 
-def dash_strip(repo):
-    return repo.strip('/')
-
-
 def user_confirm(text, valid_answers):
     try:
         answer = input(text)
@@ -71,82 +72,133 @@ def confirm_irrecoverable_operation():
                  valid_answers=['YES'])
 
 
-def cmd_print_repos(args):
-    ws = ripio.Workspace(args.owner, args.credentials)
+def cmd_ls_repos(config):
+    ws_name = ripio.WorkspaceName(config.owner)
+    if ws_name.site == 'bitbucket':
+        ws = ripio.BitbucketWorkspace(ws_name, config.credentials.get('bitbucket'))
+    elif ws_name.site == 'github':
+        ws = ripio.GithubWorkspace(ws_name, config.credentials.get('github'))
+    else:
+        raise ripio.UnsupportedSite(ws_name.site)
+
     for i, repo in enumerate(ws.ls_repos()):
         print("{0:>4}. {1:>10} - {2.scm:<3} - {2.access:<7} - {2.full_name:<20}".format(
             i+1, to_kb(repo.size), repo))
 
 
-def cmd_print_head(args):
-    repo = ripio.Repo(args.repo, args.credentials)
-    commits = repo.last_commits()
+def get_repo(config):
+    assert config.credentials
+    ws_name = ripio.RepoName(config.repo)
+    print(ws_name.site)
+    print(ws_name.full_name)
+
+    if ws_name.site == 'bitbucket':
+        return ripio.BitbucketRepo(ws_name, config.credentials.get('bitbucket'))
+    elif ws_name.site == 'github':
+        return ripio.GithubRepo(ws_name, config.credentials.get('github'))
+    
+    raise ripio.UnsupportedSite(ws_name.site)
+
+
+def cmd_print_head(config):
+    # full_name = ripio.RepoName.complete(config.repo, config)
+    repo = get_repo(config)
+    commits = list(repo.last_commits())
     if not commits:
         print("-- repository '{}' is empty".format(repo.full_name))
         return
 
     for c in commits:
-        print("{} - {}\n\t{}\n".format(
-            c['date'], c['author']['raw'], c['message'].strip()))
+        print("-- {}\n   {}\n   {}\n\n   {}".format(
+            c['hash'], c['author'], c['date'], c['message']))
 
 
-def cmd_repo_rename(args):
-    repo = ripio.Repo(args.repo, args.credentials)
-    new_name = repo.rename(args.new_name)
+def cmd_repo_rename(config):
+    repo = get_repo(config)
+    new_name = repo.rename(config.new_name)
     print("Repository '{}' renamed as '{}/{}'".format(
-        repo.full_name, ripio.owner(repo.full_name), new_name))
+        repo.name.full_name, repo.name.owner.workspace, new_name))
 
 
-def cmd_repo_create(args):
-    assert args.credentials
-    repo = ripio.Repo(args.repo, args.credentials)
-    repo.create()
-    print("Repository '{}' created".format(args.repo))
+def cmd_repo_create(config):
+    repo = get_repo(config)
+    name = repo.create()
+    print("Repository '{}' created".format(name))
 
 
-def cmd_repo_delete(args):
-    assert args.credentials
+def cmd_repo_delete(config):
+    repo = get_repo(config)
+    repo.check()
     confirm_irrecoverable_operation()
-    repo = ripio.Repo(args.repo, args.credentials)
     print("Deleting '{}'".format(repo.full_name))
     repo.delete()
 
 
-def cmd_repo_clone(args):
-    repo = ripio.Repo(args.repo, args.credentials)
-    destdir = args.destdir / repo.slug
+def cmd_repo_clone(config):
+    repo = get_repo(config)
+    destdir = config.destdir / repo.slug
+
+    if Path(destdir).exists():
+        raise ripio.DestinationDirectoryAlreadyExists(destdir)
+
     print("Cloning({}) '{}' to '{}'".format(
-        args.proto, repo.full_name, destdir))
-    repo.clone(destdir, args.proto)
+        config.proto, repo.full_name, destdir))
+    repo.clone(destdir, config.proto)
 
 
-def cmd_show_config(args):
-    config = vars(args)
+def cmd_show_config(config):
+    config = vars(config)
     del config['func']
     pprint(config)
 
 
+def cmd_site(config):
+    url = ripio.BitbucketRepo.from_dir(Path.cwd(), config.credentials).webpage
+    print("Openning '{}'".format(url))
+    webbrowser.open(url)
+
+
 def run():
-    parser = argparse.ArgumentParser(description='Manage hosted git repositories')
+    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
+        description='''\
+Manage hosted git repositories.
+General repository name format is: 'site:owner/name'.
+Examples:
+- 'github:twitter/wordpress' or 'gh:twitter/wordpress'
+- 'bitbucket:paypal/exmaple' or 'bb:paypal/exmaple'
+
+Abbreviated names are allowed when suitable configuration is given.
+''')
+
     parser.add_argument('--config', help='alternate config file')
     parser.add_argument('-c', '--credentials', type=ripio.Credentials.make,
                         help="authentication credentials with 'user:pass' format")
     parser.add_argument('-v', '--verbosity', action='count', default=0,
                         help='verbosity level. -v:INFO, -vv:DEBUG')
     cmds = parser.add_subparsers()
-    parser_ls = cmds.add_parser('ls', help='list repositories')
-    parser_ls.set_defaults(func=cmd_print_repos)
+    parser_ls = cmds.add_parser(
+        'ls', help='list repositories',
+        formatter_class=argparse.RawTextHelpFormatter,
+        description='''\
+General workspace format is: 'site:name'.
+Examples:
+- 'github:twitter'   or 'gh:twitter'
+- 'bitbucket:paypal' or 'bb:paypal'
+
+Abbreviated names are allowed when suitable configuration is given.
+''')
+    parser_ls.set_defaults(func=cmd_ls_repos)
     parser_ls.add_argument('owner', help='team or user')
 
     parser_head = cmds.add_parser('head', help='show last commits')
     parser_head.set_defaults(func=cmd_print_head)
-    parser_head.add_argument('repo', type=dash_strip,
-                            help='repo fullname: owner/slug')
+    parser_head.add_argument('repo', help='repo fullname: owner/slug')
 
     parser_rename = cmds.add_parser('rename', help='rename repository')
     parser_rename.set_defaults(func=cmd_repo_rename)
     parser_rename.add_argument('repo', help='repo fullname: owner/slug')
-    parser_rename.add_argument('new_name', metavar='new-name', help='new repository name')
+    parser_rename.add_argument('new_name', metavar='new-name',
+                               help='new repository name')
 
     parser_create = cmds.add_parser('create', help='create new repository')
     parser_create.set_defaults(func=cmd_repo_create)
@@ -168,18 +220,36 @@ def run():
     parser_config = cmds.add_parser('config', help='show config')
     parser_config.set_defaults(func=cmd_show_config)
 
-    args = parser.parse_args()
-    set_verbosity(args)
-    load_config(args)
+    parser_site = cmds.add_parser('site', help='open webpage for the current repository')
+    parser_site.set_defaults(func=cmd_site)
 
-    if not hasattr(args, 'func'):
+
+    config = parser.parse_args()
+    set_verbosity(config)
+    load_config(config)
+
+    if not hasattr(config, 'func'):
         parser.print_help()
         sys.exit(1)
 
     try:
-        args.func(args)
+        config.func(config)
+    except ripio.error as e:
+        if config.verbosity == 0:
+            print("Try 'ripio -v' for detail")
+        raise
+
+
+def main_production():
+    try:
+        run()
         print('-- ok')
     except ripio.error as e:
         print(e)
         print('-- fail')
         sys.exit(1)
+
+def main_debug():
+    run()
+
+main = main_debug
\ No newline at end of file
